/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: cfg-grammar.y,v 1.7 1999/04/10 19:13:56 bazsi Exp $
 *
 ***************************************************************************/

%{

#include <strings.h>

#include "cfgfile.h"
#include "filters.h"
#include "affile.h"
#include "afunix.h"

void yyerror(char *msg);
int yylex();

%}

%union {
	int num;
	char *cptr;
	void *ptr;
	struct filter_expr_node *node;
}

/* statements */
%token	KW_SOURCE KW_DESTINATION KW_LOG KW_OPTIONS KW_FILTER

/* source & destination items */
%token	KW_FILE KW_UNIX_STREAM KW_UNIX_DGRAM 

/* option items */
%token KW_MARK_FREQ

/* filter items*/
%token KW_FACILITY KW_PRIORITY

%token	<cptr> IDENTIFIER
%token	<cptr> FILTER_PARAMS
%token	<cptr> NUMBER
%token	<cptr> STRING
%token  <num>  COMPARE

%left	KW_OR
%left	KW_AND
%left   KW_NOT

%type	<ptr> source_stmt
%type	<ptr> dest_stmt
%type	<ptr> log_stmt
%type	<ptr> options_stmt

%type	<ptr> source_items
%type	<ptr> source_item
%type	<ptr> source_file
%type	<ptr> source_afunix

%type	<ptr> dest_items
%type	<ptr> dest_item
%type   <ptr> dest_file

%type	<ptr> log_items
%type	<ptr> log_item

%type	<ptr> options_items
%type	<ptr> options_item

%type	<ptr> filter_stmt
%type	<node> filter_expr
%type	<node> filter_simple_expr
%type	<node> filter_prio_params


%%

start   
        : stmts
	;

stmts   
        : stmt ';' stmts
	|	
	;

stmt    
        : KW_SOURCE source_stmt			{ add_source_group($2); }
	| KW_DESTINATION dest_stmt		{ add_dest_group($2); }
	| KW_LOG log_stmt			{ add_log_connection($2); }
	| KW_FILTER filter_stmt			{ add_filter_rule($2); }
	| KW_OPTIONS options_stmt		{  }
	;

source_stmt
        : IDENTIFIER '{' source_items '}'	{ $$ = make_source_group($1, $3); free($1); }
	;

dest_stmt
        : IDENTIFIER '{' dest_items '}'		{ $$ = make_dest_group($1, $3); free($1); }
	;

log_stmt
        : '{' log_items '}'			{ $$ = make_log_connection($2); }
	;

options_stmt
        : '{' options_items '}'			{ $$ = NULL; }
	;

source_items
        : source_item ';' source_items		{ append_source_driver($1, $3); $$ = $1; }
	|					{ $$ = NULL; }
	;

source_item
	: source_file				{ $$ = $1; }
	| source_afunix			{ $$ = $1; }
	;

source_file
	: KW_FILE IDENTIFIER			{ $$ = make_affile_source($2, configuration->backend); free($2); }
	;

source_afunix
	: KW_UNIX_DGRAM IDENTIFIER		{ $$ = make_afunix_source($2, AFUNIX_PROTO_DGRAM, configuration->backend); free($2); }
	| KW_UNIX_STREAM IDENTIFIER		{ $$ = make_afunix_source($2, AFUNIX_PROTO_STREAM, configuration->backend); free($2); }
	;

dest_items
	: dest_item ';' dest_items		{ append_dest_driver($1, $3); $$ = $1; }
	|					{ $$ = NULL; }
	;

dest_item
	: dest_file				{ $$ = $1; }
	;

dest_file
	: KW_FILE IDENTIFIER 			{ $$ = make_affile_dest($2, 0, configuration->backend); free($2); }
	;

log_items
	: log_item ';' log_items		{ append_endpoint_info($1, $3); $$ = $1; }
	|					{ $$ = NULL; }
	;

log_item
	: KW_SOURCE IDENTIFIER			{ $$ = make_log_endpoint_info(EP_SOURCE, $2); free($2); }
	| KW_FILTER IDENTIFIER			{ $$ = make_log_endpoint_info(EP_FILTER, $2); free($2); }
	| KW_DESTINATION IDENTIFIER		{ $$ = make_log_endpoint_info(EP_DESTINATION, $2); free($2); }
	;

options_items
	: options_item ';' options_items	{ $$ = $1; }
	|					{ $$ = NULL; }
	;

options_item
	: KW_MARK_FREQ NUMBER			{ $$ = NULL; }
	;

filter_stmt
	: IDENTIFIER '{' filter_expr ';' '}'	{ $$ = make_filter_rule($1, $3); }
	;

filter_expr
	: filter_simple_expr			{ $$ = $1; }
        | KW_NOT filter_expr			{ $2->comp = !($2->comp); $$ = $2; }
	| filter_expr KW_OR filter_expr		{ $$ = make_filter_or($1, $3); }
	| filter_expr KW_AND filter_expr	{ $$ = make_filter_and($1, $3); }
	| '(' filter_expr ')'			{ $$ = $2; }
	;

filter_simple_expr
	: KW_FACILITY '(' IDENTIFIER ')'	{ $$ = make_filter_facility($3); free($3); }
	| KW_PRIORITY filter_prio_params 	{ $$ = $2; }
	;

filter_prio_params
	: '(' IDENTIFIER ')'			{ $$ = make_filter_priority($2, 1); free($2); }
	| '(' COMPARE IDENTIFIER ')'		{ $$ = make_filter_priority($3, $2); free($3); }
	;

%%

void yyerror(char *msg)
{
	puts(msg);
}
