%{

#include "syslog-ng.h"
#include "cfg-grammar.h"

#include <string.h>

struct keyword {
	char	*kw_name;
	int	kw_token;
};

static struct keyword keywords[] = {
	/* statements */
	{ "source", 		KW_SOURCE },
	{ "destination",	KW_DESTINATION },
	{ "filter",             KW_FILTER },
	{ "log",		KW_LOG },
	{ "options",		KW_OPTIONS },
       
	/* source or destination items */
	{ "file",		KW_FILE },
	{ "mark",		KW_MARK_FREQ },
	{ "unix-dgram",		KW_UNIX_DGRAM },
	{ "unix-stream",	KW_UNIX_STREAM },

	/* filter items */
        { "or",                 KW_OR },
	{ "and",                KW_AND },
        { "not",                KW_NOT },
	{ "priority",           KW_PRIORITY },
	{ "facility",           KW_FACILITY },

	/* compare */

};

int linenum;
int lex_filter_params = 0;

static int check_reserved_words(char *token);

%}

white	[ \r\t\n]
digit	[0-9]
alpha		[a-zA-Z]
alphanum	[a-zA-Z0-9]

%%

\n			{ linenum++; }
{white}+		;
{digit}+		{ return NUMBER; }
\>\=			{ yylval.num = 1; return COMPARE; }
\>			{ yylval.num = 2; return COMPARE; }
\=\=			|
\=		        { yylval.num = 0; return COMPARE; }
\!\=			{ yylval.num = -3; return COMPARE; } 
\<\=			{ yylval.num = -1; return COMPARE; }
\<			{ yylval.num = -2; return COMPARE; }
[^ \=\<\>\(\)\{\}\\;\n\t,|]+	{ return check_reserved_words(yytext); }
\(			{ return '('; }
\)			{ return ')'; }
\;			{ return ';'; }
\{			{ return '{'; }
\}			{ return '}'; }
\|			{ return '|'; }
\,			{ return ','; }

%%

int lex_init(FILE *file)
{
	yyrestart(file);
	linenum = 0;
	return 0;
}

int check_reserved_words(char *token)
{
	int i;
	
	for (i = 0; i < (sizeof(keywords) / sizeof(struct keyword)); i++) {
		if (strcmp(keywords[i].kw_name, token) == 0) {
			return keywords[i].kw_token;
		}
	}
	yylval.cptr = strdup(token);
	return IDENTIFIER;
}
