/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: filters.c,v 1.1 1999/04/10 19:13:56 bazsi Exp $
 *
 ***************************************************************************/

#include "filters.h"
#include "format.h"
#include "syslog-names.h"
#include <syslog.h>

#define CLASS_DEFINE
#include "filters.h.x"
#undef CLASS_DEFINE

#include "filters.c.x"

struct log_filter *make_filter_rule(const char *name, struct filter_expr_node *root)
{
	NEW(log_filter, self);

	self->name = c_format("%z", name);
	self->root = root;
	return self;
}

/* CLASS:
   (class
     (name filter_expr_op)
     (super filter_expr_node)
     (vars
       (left object filter_expr_node)
       (right object filter_expr_node)))
*/

static int do_filter_and(struct filter_expr_node *c, struct log_info *log)
{
	CAST(filter_expr_op, self, c);

	return FILTER_RULE_EVAL(self->left, log) && FILTER_RULE_EVAL(self->right, log);
}

struct filter_expr_node *make_filter_and(struct filter_expr_node *l, struct filter_expr_node *r)
{
	NEW(filter_expr_op, self);
       
	self->super.eval = do_filter_and;
	self->left = l;
	self->right = r;
	return &self->super;
}

static int do_filter_or(struct filter_expr_node *c, struct log_info *log)
{
	CAST(filter_expr_op, self, c);

	return FILTER_RULE_EVAL(self->left, log) || FILTER_RULE_EVAL(self->right, log);
}
 
struct filter_expr_node *make_filter_or(struct filter_expr_node *l, struct filter_expr_node *r)
{
	NEW(filter_expr_op, self);

	self->super.eval = do_filter_or;
	self->left = l;
	self->right = r;
	return &self->super;
}

/* CLASS:
   (class
     (name filter_expr_cmp)
     (super filter_expr_node)
     (vars
       (value simple int)
       (cmp simple int)))
*/

static int do_filter_facility(struct filter_expr_node *c, struct log_info *log)
{
	CAST(filter_expr_cmp, self, c);

	return (LOG_FAC(log->pri) == self->value) ^ c->comp;
}

struct filter_expr_node *make_filter_facility(const char *fac)
{
	NEW(filter_expr_cmp, self);

	self->super.eval = do_filter_facility;
	self->value = LOG_FAC(syslog_lookup_facility(fac));
	return &self->super;
}

static int do_filter_priority(struct filter_expr_node *c, struct log_info *log)
{
	CAST(filter_expr_cmp, self, c);
	int x, y, cmp, res;

	if (self->cmp < 0) {
		y = self->value;
		x = LOG_PRI(log->pri);
		cmp = -self->cmp;
	}
	else {
		x = self->value;
		y = LOG_PRI(log->pri);
		cmp = self->cmp;
	}
	switch (cmp) {
	case 2: /* > */
		res = (x > y);
		break;
	case 1: /* >= */
		res = (x >= y);
		break;
	case 0:
		res = (x == y);
		break;
	case 3:
		res = (x != y);
		break;
	default:
		fatal("internal error, invalid compare operation %i\n", cmp);
	}

	return res ^ c->comp;
}

struct filter_expr_node *make_filter_priority(const char *prio, int cmp)
{
	NEW(filter_expr_cmp, self);

	self->super.eval = do_filter_priority;
	self->value = syslog_lookup_priority(prio);
	self->cmp = cmp;
	return &self->super;
}
