/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: syslog-names.c,v 1.1 1999/04/10 19:13:56 bazsi Exp $
 *
 ***************************************************************************/

#include "syslog-names.h"
#include "syslog-ng.h"
#include <string.h>

struct sl_name {
	char *name;
	int value;
};

static struct sl_name priorities[] = {
	{ "alert", LOG_ALERT },
	{ "crit", LOG_CRIT },
	{ "debug", LOG_DEBUG },
	{ "emerg", LOG_EMERG },
	{ "err", LOG_ERR },
	{ "error", LOG_ERR },               /* DEPRECATED */
	{ "info", LOG_INFO },
	{ "notice", LOG_NOTICE },
	{ "panic", LOG_EMERG },             /* DEPRECATED */
	{ "warn", LOG_WARNING },            /* DEPRECATED */
	{ "warning", LOG_WARNING }
}; 

static struct sl_name facilities[] = {
	{ "auth", LOG_AUTH },
	{ "authpriv", LOG_AUTHPRIV },
	{ "cron", LOG_CRON },
	{ "daemon", LOG_DAEMON },
	{ "ftp", LOG_FTP },
	{ "kern", LOG_KERN },
	{ "lpr", LOG_LPR },
	{ "mail", LOG_MAIL },
	{ "news", LOG_NEWS },
	{ "security", LOG_AUTH },           /* DEPRECATED */
	{ "syslog", LOG_SYSLOG },
	{ "user", LOG_USER },
	{ "uucp", LOG_UUCP },
	{ "local0", LOG_LOCAL0 },
	{ "local1", LOG_LOCAL1 },
	{ "local2", LOG_LOCAL2 },
	{ "local3", LOG_LOCAL3 },
	{ "local4", LOG_LOCAL4 },
	{ "local5", LOG_LOCAL5 },
	{ "local6", LOG_LOCAL6 },
	{ "local7", LOG_LOCAL7 }
};

struct sl_name *syslog_lookup_name(const char *name, struct sl_name names[], size_t num)
{
	int i;

	for (i = 0; i < num; i++) {
		if (strcasecmp(name, names[i].name) == 0) {
			return &names[i];
		}
	}
	return NULL;
}

int syslog_lookup_priority(const char *name)
{
	struct sl_name *entry = syslog_lookup_name(name, priorities, sizeof(priorities) / sizeof(struct sl_name));
	return entry ? entry->value : -1;
}

int syslog_lookup_facility(const char *name)
{
	struct sl_name *entry = syslog_lookup_name(name, facilities, sizeof(facilities) / sizeof(struct sl_name));
	return entry ? entry->value : -1;
}
